import numpy as np
import pytest

import fast_stats


@pytest.mark.parametrize(
    "y_true,y_pred,zero_division,expected",
    [
        (
            np.zeros(4, dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            "zero",
            0,
        ),  # all FP
        (
            np.ones(4, dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            "zero",
            1.0,
        ),  # all TP
        (
            np.zeros(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "zero",
            0.0,
        ),  # No TP & No FP
        (
            np.zeros(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "none",
            None,
        ),  # No TP & No FP
        (
            np.ones(4, dtype=np.uint64),
            np.array([1, 0, 0, 0], dtype=np.uint64),
            "none",
            1.0,
        ),  # 1 TP & 0 FP
        (
            np.array([1, 0, 0, 0], dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            "none",
            0.25,
        ),  # 1 TP & 3 FP
        (
            np.zeros(4, dtype=np.uint64),
            np.array([1, 0, 0, 0], dtype=np.uint64),
            "none",
            0.0,
        ),  # 0 TP & 1 TP
    ],
)
def test_precision(y_true, y_pred, zero_division, expected):
    assert fast_stats.binary_precision(y_true, y_pred, zero_division) == expected


@pytest.mark.parametrize(
    "y_true,y_pred,zero_division,expected",
    [
        (
            np.ones(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "zero",
            0,
        ),  # all FN
        (
            np.ones(4, dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            "zero",
            1.0,
        ),  # all TP
        (
            np.zeros(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "zero",
            0.0,
        ),  # No TP & No FN
        (
            np.zeros(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "none",
            None,
        ),  # No TP & No FN
        (
            np.ones(4, dtype=np.uint64),
            np.array([1, 0, 0, 0], dtype=np.uint64),
            "none",
            0.25,
        ),  # 1 TP & 3 FN
        (
            np.array([1, 0, 0, 0], dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            "none",
            1.0,
        ),  # 1 TP & 0 FN
        (
            np.array([1, 0, 0, 0], dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "none",
            0.0,
        ),  # 0 TP & 1 FN
    ],
)
def test_recall(y_true, y_pred, zero_division, expected):
    assert fast_stats.binary_recall(y_true, y_pred, zero_division) == expected


@pytest.mark.parametrize(
    "y_true,y_pred,zero_division,expected",
    [
        (
            np.ones(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "zero",
            0.0,
        ),  # all FN
        (
            np.ones(4, dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            "zero",
            1.0,
        ),  # all TP
        (
            np.zeros(4, dtype=np.uint64),
            np.zeros(4, dtype=np.uint64),
            "zero",
            0.0,
        ),  # No TP & No FP, & No FN
        (
            np.ones(4, dtype=np.uint64),
            np.array([1, 0, 0, 0], dtype=np.uint64),
            "none",
            2 * (1 / 4 * 1.0) / (1 / 4 + 1.0),
        ),  # 1 TP & 3 FN
        (
            np.array([1, 1, 0, 0], dtype=np.uint64),
            np.array([0, 1, 1, 0], dtype=np.uint64),
            "none",
            0.5,
        ),
    ],
)
def test_f1(y_true, y_pred, zero_division, expected):
    assert np.allclose(
        fast_stats.binary_f1_score(y_true, y_pred, zero_division), expected
    )



import numpy as np
import pytest

from fast_stats import confusion_matrix


@pytest.mark.parametrize(
    "y_true,y_pred,expected",
    [
        (
            np.ones(4, dtype=np.uint64),
            np.ones(4, dtype=np.uint64),
            np.array([[4]]),
        ),  # all one value
        (
            np.array(
                [
                    [1, 2],
                    [1, 2],
                ]
            ),
            np.array(
                [
                    [1, 1],
                    [2, 2],
                ]
            ),
            np.ones((2, 2)),
        ),  # 2x2
        (
            np.array(
                [
                    [1, 2, 3],
                    [1, 2, 3],
                    [1, 2, 3],
                ]
            ),
            np.array(
                [
                    [1, 2, 1],
                    [1, 1, 1],
                    [1, 1, 3],
                ]
            ),
            np.array(
                [
                    [3, 0, 0],
                    [2, 1, 0],
                    [2, 0, 1],
                ]
            ),
        ),  # 3x3
    ],
)
def test_precision(y_true, y_pred, expected):
    assert np.allclose(confusion_matrix(y_true, y_pred), expected)



from ._fast_stats_ext import (
    _binary_f1_score_reqs,
    _binary_precision_reqs,
    _binary_recall_reqs,
    _confusion_matrix,
    _unique,
)
from .binary import binary_f1_score, binary_precision, binary_recall
from .confusion_matrix import confusion_matrix



from typing import Union

import numpy as np

from ._fast_stats_ext import (
    _binary_f1_score_reqs,
    _binary_precision_reqs,
    _binary_recall_reqs,
)

Result = Union[None, float]


def _precision(tp: int, tp_fp: int, zero_division: str = "none") -> Result:
    if tp_fp == 0:
        if zero_division == "none":
            return None
        elif zero_division == "zero":
            return 0.0
    return tp / tp_fp


def _recall(tp: int, tp_fn: int, zero_division: str = "none") -> Result:
    if tp_fn == 0:
        if zero_division == "none":
            return None
        elif zero_division == "zero":
            return 0.0
    return tp / tp_fn


def binary_precision(
    y_true: np.ndarray, y_pred: np.ndarray, zero_division: str = "none"
) -> Result:
    assert y_true.shape == y_pred.shape, "y_true and y_pred must be same shape"
    assert all(
        [
            isinstance(y_pred, np.ndarray),
            isinstance(y_true, np.ndarray),
        ]
    ), "y_true and y_pred must be numpy arrays"

    tp, tp_fp, _ = _binary_precision_reqs(y_true, y_pred)
    return _precision(tp, tp_fp, zero_division)


def binary_recall(
    y_true: np.ndarray, y_pred: np.ndarray, zero_division: str = "none"
) -> Result:
    assert y_true.shape == y_pred.shape, "y_true and y_pred must be same shape"
    assert all(
        [
            isinstance(y_pred, np.ndarray),
            isinstance(y_true, np.ndarray),
        ]
    ), "y_true and y_pred must be numpy arrays"

    tp, tp_fn, _ = _binary_recall_reqs(y_true, y_pred)
    return _recall(tp, tp_fn, zero_division)


def binary_f1_score(
    y_true: np.ndarray, y_pred: np.ndarray, zero_division: str = "none"
) -> Result:
    assert y_true.shape == y_pred.shape, "y_true and y_pred must be same shape"
    assert all(
        [
            isinstance(y_pred, np.ndarray),
            isinstance(y_true, np.ndarray),
        ]
    ), "y_true and y_pred must be numpy arrays"

    tp, tp_fp, tp_fn = _binary_f1_score_reqs(y_true, y_pred)
    p, r = _precision(tp, tp_fp, "zero"), _recall(tp, tp_fn, "zero")

    if p + r == 0:
        if zero_division == "none":
            return None
        elif zero_division == "zero":
            return 0.0

    return 2 * p * r / (p + r)



from typing import List

import numpy as np

from ._fast_stats_ext import _confusion_matrix, _unique


def confusion_matrix(
    y_true: np.ndarray, y_pred: np.ndarray, labels: List = None
) -> np.ndarray:
    assert y_true.shape == y_pred.shape, "y_true and y_pred must be same shape"
    assert all(
        [
            isinstance(y_pred, np.ndarray),
            isinstance(y_true, np.ndarray),
        ]
    ), "y_true and y_pred must be numpy arrays"

    if labels is None:
        labels = sorted(list(_unique(np.concatenate([y_true, y_pred]))))

    return _confusion_matrix(y_true, y_pred, labels)
